@@ -1,8 +1,11 @@
-from datetime import date, datetime
+from datetime import date, datetime, timedelta
 from os.path import abspath, dirname, join
 from pathlib import Path
 from typing import IO, TYPE_CHECKING, Any, BinaryIO, Dict, Iterable, List, Optional, Tuple, Union
 
+from appdirs import user_cache_dir
+from dateutil.relativedelta import relativedelta
+
 # iNaturalist URLs
 API_V0_BASE_URL = 'https://www.inaturalist.org'
 API_V1_BASE_URL = 'https://api.inaturalist.org/v1'
@@ -18,29 +21,41 @@ PHOTO_INFO_BASE_URL = 'https://www.inaturalist.org/photos'
 # Prefix used for keyring entries
 KEYRING_KEY = '/inaturalist'
 
-# Pagination and rate-limiting values
+# Pagination settings
 PER_PAGE_RESULTS = 200  # Default number of records per page for paginated queries
-THROTTLING_DELAY = 1.0  # Delay between paginated queries, in seconds
-MAX_DELAY = 60  # Maximum time to wait for rate-limiting before aborting
-REQUESTS_PER_SECOND = 2
-REQUESTS_PER_MINUTE = 60
-REQUESTS_PER_DAY = 10000
 LARGE_REQUEST_WARNING = 5000  # Show a warning for queries that will return over this many results
 
-PHOTO_SIZES = ['square', 'small', 'medium', 'large', 'original']
+# Rate-limiting and retry settings
+MAX_DELAY = 60  # Maximum time to wait for rate-limiting before aborting
+REQUEST_BURST_RATE = 5
+REQUESTS_PER_SECOND = 1
+REQUESTS_PER_MINUTE = 60
+REQUESTS_PER_DAY = 10000
+REQUEST_TIMEOUT = 10
+REQUEST_RETRIES = 5  # Maximum number of retries for a failed request
+RETRY_BACKOFF = 0.5  # Exponential backoff factor for retries
 
-# Toggle dry-run mode: this will run and log mock HTTP requests instead of real ones
-DRY_RUN_ENABLED = False  # Mock all requests, including GET
-DRY_RUN_WRITE_ONLY = False  # Only mock 'write' requests
+# HTTP methods that apply to write-only dry-run mode
 WRITE_HTTP_METHODS = ['PATCH', 'POST', 'PUT', 'DELETE']
 
 # Project directories
 PROJECT_DIR = abspath(dirname(dirname(__file__)))
+CACHE_DIR = user_cache_dir('pyinaturalist')
 DOCS_DIR = join(PROJECT_DIR, 'docs')
 DOWNLOAD_DIR = join(PROJECT_DIR, 'downloads')
 EXAMPLES_DIR = join(PROJECT_DIR, 'examples')
 SAMPLE_DATA_DIR = join(PROJECT_DIR, 'test', 'sample_data')
 
+# Cache settings
+CACHE_EXPIRATION = {
+    'api.inaturalist.org/v*/controlled_terms*': timedelta(days=1),
+    'api.inaturalist.org/v*/places*': timedelta(days=1),
+    'api.inaturalist.org/v*/taxa*': timedelta(days=1),
+    '*': timedelta(minutes=30),
+}
+CACHE_FILE = join(CACHE_DIR, 'api_requests.db')
+TOKEN_EXPIRATION = timedelta(hours=1)
+
 # Response formats supported by GET /observations endpoint
 OBSERVATION_FORMATS = ['atom', 'csv', 'dwc', 'json', 'kml', 'widget']
 
@@ -64,7 +79,6 @@ ICONIC_TAXA = {
     48222: 'Chromista',
     47686: 'Protozoa',
 }
-# TODO: More emoji for non-iconic taxa?
 ICONIC_EMOJI = {
     0: '❓',
     1: '🐾',
@@ -112,11 +126,6 @@ RANKS = [
     'kingdom',
 ]
 
-# Endpoint-specific options for multiple choice parameters
-NODE_OBS_ORDER_BY_PROPERTIES = ['created_at', 'id', 'observed_on', 'species_guess', 'votes']
-REST_OBS_ORDER_BY_PROPERTIES = ['date_added', 'observed_on']
-PROJECT_ORDER_BY_PROPERTIES = ['created', 'distance', 'featured', 'recent_posts', 'updated']
-
 # Options for multiple choice parameters (non-endpoint-specific)
 CC_LICENSES = ['CC-BY', 'CC-BY-NC', 'CC-BY-ND', 'CC-BY-SA', 'CC-BY-NC-ND', 'CC-BY-NC-SA', 'CC0']
 ALL_LICENSES = CC_LICENSES + ['ALL RIGHTS RESERVED']
@@ -128,15 +137,23 @@ HAS_PROPERTIES = ['photo', 'geo']
 HISTOGRAM_DATE_FIELDS = ['created', 'observed']
 HISTOGRAM_INTERVALS = ['year', 'month', 'week', 'day', 'hour', 'month_of_year', 'week_of_year']
 ID_CATEGORIES = ['improving', 'supporting', 'leading', 'maverick']
+INBOXES = ['inbox', 'sent', 'any']
 ORDER_DIRECTIONS = ['asc', 'desc']
+PHOTO_SIZES = ['square', 'small', 'medium', 'large', 'original']
 PLACE_CATEGORIES = ['standard', 'community']
 PROJECT_TYPES = ['assessment', 'bioblitz', 'collection', 'umbrella']
 QUALITY_GRADES = ['casual', 'needs_id', 'research']
 SEARCH_PROPERTIES = ['names', 'tags', 'description', 'place']
 SOURCES = ['places', 'projects', 'taxa', 'users']
 
+# Endpoint-specific options for multiple choice parameters
+V0_OBS_ORDER_BY_PROPERTIES = ['date_added', 'observed_on']
+V1_OBS_ORDER_BY_PROPERTIES = ['created_at', 'id', 'observed_on', 'species_guess', 'votes']
+PROJECT_ORDER_BY_PROPERTIES = ['created', 'distance', 'featured', 'recent_posts', 'updated']
+
 # Multiple-choice request parameters, with keys mapped to their possible choices (non-endpoint-specific)
 MULTIPLE_CHOICE_PARAMS = {
+    'box': INBOXES,
     'category': ID_CATEGORIES,
     'csi': CONSERVATION_STATUSES,
     'date_field': HISTOGRAM_DATE_FIELDS,
@@ -165,7 +182,7 @@ MULTIPLE_CHOICE_PARAMS = {
     'type': PROJECT_TYPES,
 }
 
-# All request parameters from both Node API and REST (Rails) API that accept date or datetime strings
+# Request parameters from all API versions that accept date or datetime strings
 DATETIME_PARAMS = [
     'created_after',
     'created_d1',
@@ -183,8 +200,9 @@ DATETIME_PARAMS = [
     'older_than',
     'on',
     'since',
-    'updated_since',  # TODO: test if this one behaves differently in Node API vs REST API
+    'updated_since',
 ]
+DATETIME_SHORT_FORMAT = '%b %d, %Y'
 
 # Type aliases
 Coordinates = Tuple[float, float]
@@ -193,6 +211,9 @@ AnyDateTime = Union[datetime, str]
 AnyFile = Union[IO, Path, str]
 DateTime = datetime
 DateOrInt = Union[date, datetime, int]
+DateOrStr = Union[date, datetime, str]
+DateOrDatetime = Union[date, datetime]
+DateRange = Tuple[DateOrDatetime, DateOrDatetime]
 Dimensions = Tuple[int, int]
 FileOrPath = Union[BinaryIO, str]
 GeoJson = Dict[str, Any]
@@ -211,6 +232,7 @@ MultiStr = Union[str, Iterable[str]]
 MultiIntOrStr = Union[MultiInt, MultiStr]
 TableRow = Dict[str, Any]
 TemplateFunction = Any  # Cannot use Callable/Protocol, as these will not allow a mix of signatures
+TimeInterval = Union[str, timedelta, relativedelta]
 
 # For type checking purposes, some nullable attrs need to be marked as Optional.
 # For documentation purposes, this is redundant since all keyword args are optional.
