@@ -1,8 +1,6 @@
 """Type conversion utilities"""
+import re
 from datetime import date, datetime, timedelta
-from dateutil.parser import UnknownTimezoneWarning  # type: ignore  # (missing from type stubs)
-from dateutil.parser import parse as parse_date
-from dateutil.tz import tzlocal, tzoffset
 from io import BytesIO
 from logging import getLogger
 from os.path import abspath, expanduser
@@ -10,6 +8,11 @@ from pathlib import Path
 from typing import IO, Any, Dict, List, MutableSequence, Optional, Union
 from warnings import catch_warnings, simplefilter
 
+from dateutil.parser import UnknownTimezoneWarning  # type: ignore  # (missing from type stubs)
+from dateutil.parser import parse as parse_date
+from dateutil.tz import tzlocal, tzoffset
+from requests import Session
+
 from pyinaturalist.constants import (
     AnyFile,
     Coordinates,
@@ -29,6 +32,10 @@ OBSERVATION_TIME_FIELDS = (
     'observed_time_zone',
     'time_zone_offset',
 )
+
+# Extremely simplified URL regex, just enough to differentiate from local paths
+URL_PATTERN = re.compile(r'^https?://.+')
+
 logger = getLogger(__name__)
 
 
@@ -83,12 +90,10 @@ def convert_histogram(response: JsonResponse) -> HistogramResponse:
 
 def convert_csv_list(obj: Any) -> str:
     """Convert list parameters into an API-compatible (comma-delimited) string"""
-    if not obj:
-        return ''
-    elif isinstance(obj, list):
+    if isinstance(obj, list):
         return ','.join(map(str, obj))
     else:
-        return str(obj)
+        return obj
 
 
 def convert_isoformat(value: Union[date, datetime, str]) -> str:
@@ -142,6 +147,9 @@ def convert_generic_timestamps(result: ResponseResult) -> ResponseResult:
     **Note:** Compared to observation timestamps, these are generally more reliable. These seem to
     be consistently in ISO 8601 format.
     """
+    if not result:
+        return result
+
     # Format inner record if present, e.g. for search results
     if 'record' in result:
         result['record'] = convert_generic_timestamps(result['record'])
@@ -255,11 +263,31 @@ def strip_empty_values(values: Dict) -> Dict:
     return {k: v for k, v in values.items() if v or v in [False, 0, 0.0]}
 
 
-# Type conversion functions for collections
+# Type conversion functions for files and collections
 # -------------------
 
 
-def ensure_list(value: Any, convert_csv: bool = False, delimiter: str = ',') -> MutableSequence[Any]:
+def ensure_file_obj(value: AnyFile, session: Session = None) -> IO:
+    """Given a file path or URL, load data into a file-like object"""
+    # Load from URL
+    if isinstance(value, str) and URL_PATTERN.match(value):
+        session = session or Session()
+        return session.get(value).raw
+
+    # Load from local file path
+    if isinstance(value, (str, Path)):
+        file_path = abspath(expanduser(value))
+        logger.info(f'Reading from file: {file_path}')
+        with open(file_path, 'rb') as f:
+            return BytesIO(f.read())
+
+    # Otherwise, assume it's already a file or file-like object
+    return value
+
+
+def ensure_list(
+    value: Any, convert_csv: bool = False, delimiter: str = ','
+) -> MutableSequence[Any]:
     """Convert an object, response, or (optionally) comma-separated string into a list"""
     if not value:
         return []
@@ -276,16 +304,6 @@ def ensure_list(value: Any, convert_csv: bool = False, delimiter: str = ',') ->
         return [value]
 
 
-def ensure_file_obj(value: AnyFile) -> IO:
-    """Given a file object or path, read it into a file-like object if it's a path"""
-    if isinstance(value, (str, Path)):
-        file_path = abspath(expanduser(value))
-        logger.info(f'Reading from file: {file_path}')
-        with open(file_path, 'rb') as f:
-            return BytesIO(f.read())
-    return value
-
-
 # Formatting functions
 # --------------------
 
