@@ -1,12 +1,19 @@
+"""Reusable template functions used for API documentation.
+Each template function contains all or part of an endpoint's request parameters, type annotations,
+and docstrings. These are then applied with :py:func:`.copy_doc_signature`.
+
+This is intended to reduce large amounts of duplicated function signatures and corresponding
+docstrings. If you're a contributor looking to add a new endpoint, using these templates isn't
+necessary. If the new function(s) have a large number of parameters and/or have many parameters in
+common with other functions, consider using a template function.
+
+Note: Since the templates are applied dynamically at import time, this adds a tiny amount of overhead
+(about 20 milliseconds as of v0.14) to the import time of the library. If this ever becomes a problem,
+an option could be added to disable ``copy_doc_signature``, e.g. via environment variable.
 """
-Reusable template functions used for API documentation.
-Each template function contains a portion of an endpoint's request parameters, with corresponding
-type annotations and docstrings.
-"""
-from typing import List
+from typing import Any, List
 
 from pyinaturalist.constants import (
-    MULTIPLE_CHOICE_PARAMS,
     AnyDate,
     AnyDateTime,
     IntOrStr,
@@ -55,7 +62,7 @@ def _identification_params(
     id_above: int = None,
     id_below: int = None,
 ):
-    """
+    """Args:
     current_taxon: ID's taxon is the same it's observation's taxon
     own_observation: ID was added by the observer
     is_change: ID was created as a results of a taxon change
@@ -91,11 +98,38 @@ def _identification_params(
     """
 
 
+# Messages
+# --------------------
+
+
+def _message_params(
+    page: int = None,
+    box: str = None,
+    q: str = None,
+    user_id: int = None,
+    threads: bool = False,
+):
+    """Args:
+    page: Page number of the results to return
+    box: One of 'inbox', 'sent', or 'any'
+    q: Search string for message subject and body
+    user_id: Get messages to/from this user
+    threads: Group results by ``thread_id``, and only get the latest message per thread.
+        Incompatible with ``q`` param.
+    """
+
+
+def _message_id(message_id: int):
+    """Args:
+    message_id: Get the message with this ID. Multiple IDs are allowed.
+    """
+
+
 # Observations
 # --------------------
 
 
-# Params that are in most observation-related endpoints in both Node and REST APIs
+# Params that are in most observation-related endpoints in all API versions
 def _observation_common(
     q: str = None,
     d1: AnyDate = None,
@@ -114,7 +148,7 @@ def _observation_common(
     iconic_taxa: MultiStr = None,
     updated_since: AnyDateTime = None,
 ):
-    """
+    """Args:
     q: Search observation properties
     d1: Must be observed on or after this date
     d2: Must be observed on or before this date
@@ -134,8 +168,8 @@ def _observation_common(
     """
 
 
-# Observation params that are only in the Node API
-def _observation_node_only(
+# Observation params that are only in the v1 API
+def _observation_v1(
     acc: bool = None,
     captive: bool = None,
     endemic: bool = None,
@@ -198,7 +232,7 @@ def _observation_node_only(
     preferred_place_id: int = None,
     ttl: str = None,
 ):
-    """
+    """Args:
     acc: Whether or not positional accuracy / coordinate uncertainty has been specified
     captive: Captive or cultivated observations
     endemic: Observations whose taxa are endemic to their location
@@ -274,8 +308,8 @@ def _observation_node_only(
     """
 
 
-# Observation params that are only in the REST API
-def _observation_rest_only(
+# Observation params that are only in the v0 API
+def _observation_v0(
     has: MultiStr = None,
     on: AnyDate = None,
     m1: AnyDate = None,
@@ -285,7 +319,7 @@ def _observation_rest_only(
     extra: str = None,
     response_format: str = 'json',
 ):
-    """
+    """Args:
     has: Catch-all for some boolean selectors. This can be used multiple times, e.g.
         ``has=['photos', 'geo']``
     m1: First month of a month range
@@ -306,12 +340,20 @@ def _observation_histogram(
     date_field: str = 'observed',
     interval: str = 'month_of_year',
 ):
-    """
+    """Args:
     date_field: Histogram basis: either when the observation was created or observed
     interval: Time interval for histogram, with groups starting on or contained within the group value.
     """
 
 
+def _ofvs(observation_id: int, observation_field_id: int, value: Any):
+    """Args:
+    observation_id: ID of the observation receiving this observation field value
+    observation_field_id: ID of the observation field for this observation field value
+    value: Value for the observation field
+    """
+
+
 def _create_observation(
     species_guess: str = None,
     taxon_id: int = None,
@@ -334,8 +376,9 @@ def _create_observation(
     local_photos: MultiFile = None,
     photos: MultiFile = None,
     sounds: MultiFile = None,
+    photo_ids: MultiIntOrStr = None,
 ):
-    """
+    """Args:
     species_guess: Equivalent to the 'What did you see?' field on the observation form.
         iNat will try to choose a single taxon based on this, but it may fail if it's ambuguous
     taxon_id: ID of the taxon to associate with this observation
@@ -360,8 +403,9 @@ def _create_observation(
         their Picasa and iNat accounts connected, and the user must own the photo(s) on Picasa.
     facebook_photos: Facebook photo IDs to add as photos for this observation. User must have
         their Facebook and iNat accounts connected, and the user must own the photo on Facebook.
-    photos: Image files, file-like objects, and/or paths
-    sounds: Sound files, file-like objects, and/or pathss
+    photos: One or more image files, file-like objects, file paths, or URLs
+    sounds: One or more sound files, file-like objects, file paths, or URLs
+    photo_ids: One or more IDs of previously uploaded photos to attach to the observation
     """
 
 
@@ -369,10 +413,21 @@ def _update_observation(
     # _method: str = None,  # Exposed as a client-specific workaround; not needed w/ `requests`
     ignore_photos: bool = True,
 ):
+    """Args:
+    ignore_photos: If photos exist on the observation but are missing in the request, ignore them instead of deleting the missing observation photos
     """
-    ignore_photos
-        If photos exist on the observation but are missing in the request, simply ignore them
-        instead of deleting the missing observation photos
+
+
+# Posts
+# --------------------
+
+
+def _get_posts(login: str = None, project_id: int = None, page: int = None, per_page: int = None):
+    """Args:
+    login: Return posts by this user
+    project_id: Return posts from this project
+    page: Pagination page number
+    per_page: Number of results to return in a page. The maximum value is generally 200 unless otherwise noted
     """
 
 
@@ -398,7 +453,7 @@ def _projects_params(
     has_params: bool = None,
     has_posts: bool = None,
 ):
-    """
+    """Args:
     q: Project name must begin with this value
     id: Must have this ID
     not_id: Must not have this ID
@@ -422,6 +477,14 @@ def _projects_params(
     """
 
 
+def _project_observation_params(access_token: str, project_id: int, observation_id: int):
+    """Args:
+    access_token: An access token required for user authentication, as returned by :py:func:`.get_access_token()`
+    project_id: ID of project to add onto
+    observation_id: ID of observation to add
+    """
+
+
 # Main Search
 # --------------------
 
@@ -433,7 +496,7 @@ def _search_params(
     locale: str = None,
     preferred_place_id: int = None,
 ):
-    """
+    """Args:
     q: Search query
     sources: Object types to search
     place_id: Results must be associated with this place
@@ -458,7 +521,7 @@ def _taxon_params(
     preferred_place_id: int = None,
     all_names: bool = None,
 ):
-    """
+    """Args:
     q: Name must begin with this value
     is_active: Taxon is active
     taxon_id: Only show taxa with this ID, or its descendants
@@ -479,7 +542,7 @@ def _taxon_id_params(
     only_id: int = None,
     parent_id: int = None,
 ):
-    """
+    """Args:
     id_above: Must have an ID above this value
     id_below: Must have an ID below this value
     only_id: Return only the record IDs
@@ -487,27 +550,13 @@ def _taxon_id_params(
     """
 
 
-# Posts
-# --------------------
-
-
-def _get_posts(login: str = None, project_id: int = None, page: int = None, per_page: int = None):
-    """
-    login: Return posts by this user
-    project_id: Return posts from this project
-    page: Pagination page number
-    per_page: Number of results to return in a page. The maximum value is generally 200 unless otherwise noted
-    """
-
-
 # Individual/common params
 # ------------------------
 
 
 def _access_token(access_token: str = None):
-    """
-    access_token: An access token required for user authentication, as returned by
-        :py:func:`get_access_token()`
+    """Args:
+    access_token: An access token required for user authentication, as returned by :py:func:`.get_access_token()`
     """
 
 
@@ -517,7 +566,7 @@ def _bounding_box(
     swlat: float = None,
     swlng: float = None,
 ):
-    """
+    """Args:
     nelat: NE latitude of bounding box
     nelng: NE longitude of bounding box
     swlat: SW latitude of bounding box
@@ -526,31 +575,31 @@ def _bounding_box(
 
 
 def _geojson_properties(properties: List[str] = None):
-    """
+    """Args:
     properties: Properties from observation results to include as GeoJSON properties
     """
 
 
 def _name(name: str = None):
-    """
+    """Args:
     name: Name must match this value
     """
 
 
 def _only_id(only_id: bool = False):
-    """
+    """Args:
     only_id: Return only the record IDs
     """
 
 
 def _observation_id(observation_id: int):
-    """
+    """Args:
     observation_id: iNaturalist observation ID
     """
 
 
 def _project_id(observation_id: int = None):
-    """
+    """Args:
     project_id: Only show users who are members of this project
     """
 
@@ -561,32 +610,27 @@ def _pagination(
     order: str = None,
     order_by: str = None,
     count_only: bool = None,
+    reverse: bool = None,
 ):
-    """
+    """Args:
     page: Page number of results to return
     per_page: Number of results to return in a page. The maximum value is generally 200,
         unless otherwise noted
     order: Sort order
     order_by: Field to sort on
     count_only: Only return a count of results; alias for ``per_page=0``
+    reverse: Reverse the order of results; alias for ``order='descending'``
     """
 
 
 _get_observations = [
     _observation_common,
-    _observation_node_only,
+    _observation_v1,
     _bounding_box,
 ]
 
 
 def _search_query(q: str = None):
-    """
+    """Args:
     q: Search query
     """
-
-
-def _format_param_choices():
-    return '\n'.join([f'  * {param}: {choices}' for param, choices in MULTIPLE_CHOICE_PARAMS.items()])
-
-
-MULTIPLE_CHOICE_PARAM_DOCS = '**Multiple-Choice Parameters:**\n' + _format_param_choices()
