@@ -10,12 +10,12 @@ from attr import define, validators
 
 from pyinaturalist.constants import JsonResponse, ResponseOrResults
 from pyinaturalist.converters import convert_lat_long, try_datetime
-from pyinaturalist.models.base import BaseModel, BaseModelCollection, load_json
+from pyinaturalist.models.base import BaseModel, BaseModelCollection, T, load_json
 from pyinaturalist.models.lazy_property import (
     LazyProperty,
     add_lazy_attrs,
+    get_lazy_attrs,
     get_lazy_properties,
-    get_model_fields,
 )
 
 
@@ -71,18 +71,19 @@ def upper(value) -> Optional[str]:
 
 from pyinaturalist.models.photo import Photo
 from pyinaturalist.models.place import Place
-from pyinaturalist.models.user import User
-from pyinaturalist.models.taxon import (
+from pyinaturalist.models.user import User, UserCount, UserCounts
+from pyinaturalist.models.taxon_meta import (
     ConservationStatus,
     EstablishmentMeans,
-    Taxon,
-    TaxonCount,
-    TaxonCounts,
+    ListedTaxon,
+    TaxonSummary,
 )
+from pyinaturalist.models.taxon import Taxon, TaxonCount, TaxonCounts
 from pyinaturalist.models.controlled_term import Annotation, ControlledTerm, ControlledTermValue
 from pyinaturalist.models.comment import Comment
 from pyinaturalist.models.identification import Identification
 from pyinaturalist.models.life_list import LifeList, LifeListTaxon
+from pyinaturalist.models.message import Message
 from pyinaturalist.models.observation_field import ObservationField, ObservationFieldValue
 from pyinaturalist.models.project import (
     Project,
