@@ -6,6 +6,7 @@ from attr import define
 
 from pyinaturalist.constants import (
     ALL_LICENSES,
+    DATETIME_SHORT_FORMAT,
     GEOPRIVACY_LEVELS,
     QUALITY_GRADES,
     Coordinates,
@@ -13,6 +14,7 @@ from pyinaturalist.constants import (
     TableRow,
 )
 from pyinaturalist.converters import convert_observation_timestamp
+from pyinaturalist.docs import extend_init_signature
 from pyinaturalist.models import (
     Annotation,
     BaseModel,
@@ -33,13 +35,12 @@ from pyinaturalist.models import (
     field,
     upper,
 )
-from pyinaturalist.v1 import get_observation
 
 
 @define(auto_attribs=False, init=False, field_transformer=add_lazy_attrs)
 class Observation(BaseModel):
     """:fa:`binoculars` An observation, based the schema of
-    `GET /observations <https://api.inaturalist.org/v1/docs/#!/Observations/get_observations>`_.
+    :v1:`GET /observations <Observations/get_observations>`
     """
 
     created_at: datetime = datetime_now_field(doc='Date and time the observation was created')
@@ -48,7 +49,9 @@ class Observation(BaseModel):
     )
     community_taxon_id: int = field(default=None, doc='The current community identification taxon')
     description: str = field(default=None, doc='Observation description')
-    faves: List[Dict] = field(factory=list, doc='Details on users who have favorited the observation')
+    faves: List[Dict] = field(
+        factory=list, doc='Details on users who have favorited the observation'
+    )
     geoprivacy: str = field(default=None, options=GEOPRIVACY_LEVELS, doc='Location privacy level')
     identifications_count: int = field(default=None, doc='Total number of identifications')
     identifications_most_agree: bool = field(
@@ -110,12 +113,16 @@ class Observation(BaseModel):
     )
     quality_grade: str = field(default=None, options=QUALITY_GRADES, doc='Quality grade')
     quality_metrics: List[Dict] = field(factory=list, doc='Data quality assessment metrics')
-    reviewed_by: List[int] = field(factory=list, doc='IDs of users who have reviewed the observation')
+    reviewed_by: List[int] = field(
+        factory=list, doc='IDs of users who have reviewed the observation'
+    )
     site_id: int = field(
         default=None, doc='Site ID for iNaturalist network members, or ``1`` for inaturalist.org'
     )
     sounds: List[Dict] = field(factory=list, doc='Observation sound files')
-    species_guess: str = field(default=None, doc="Taxon name from observer's initial identification")
+    species_guess: str = field(
+        default=None, doc="Taxon name from observer's initial identification"
+    )
     tags: List[str] = field(factory=list, doc='Arbitrary user tags added to the observation')
     updated_at: DateTime = datetime_field(doc='Date and time the observation was last updated')
     uri: str = field(default=None, doc='Link to observation details page')
@@ -139,7 +146,9 @@ class Observation(BaseModel):
         type=List[ObservationFieldValue],
         doc='Observation field values',
     )
-    photos: property = LazyProperty(Photo.from_json_list, type=List[Photo], doc='Observation photos')
+    photos: property = LazyProperty(
+        Photo.from_json_list, type=List[Photo], doc='Observation photos'
+    )
     project_observations: property = LazyProperty(
         ProjectObservation.from_json_list,
         type=List[ProjectObservation],
@@ -175,8 +184,7 @@ class Observation(BaseModel):
         'time_zone_offset',
     ]
 
-    # Convert observation timestamps prior to attrs init
-    # TODO: better function signature for docs; use forge?
+    # Convert observation timestamps prior to __attrs_init__
     def __init__(
         self,
         created_at_details: Dict = None,
@@ -201,6 +209,8 @@ class Observation(BaseModel):
     @classmethod
     def from_id(cls, id: int):
         """Lookup and create a new Observation object from an ID"""
+        from pyinaturalist.v1 import get_observation
+
         json = get_observation(id)
         return cls.from_json(json)
 
@@ -232,8 +242,8 @@ class Observation(BaseModel):
     def __str__(self) -> str:
         return (
             f'[{self.id}] {self.taxon.full_name} '
-            f'observed on {self.observed_on} by {self.user.login} '
-            f'at {self.place_guess or self.location}'
+            f'observed on {self.observed_on.strftime(DATETIME_SHORT_FORMAT)} '
+            f'by {self.user.login} at {self.place_guess or self.location}'
         )
 
 
@@ -265,3 +275,7 @@ class Observations(BaseModelCollection):
     def thumbnail_urls(self) -> List[str]:
         """Get thumbnails for all observation default photos"""
         return [obs.thumbnail_url for obs in self.data if obs.thumbnail_url]
+
+
+# Fix __init__ and class docstring
+Observation = extend_init_signature(Observation.__attrs_init__)(Observation)  # type: ignore
