@@ -48,16 +48,17 @@ class Place(BaseModel):
         doc='Location in ``(latitude, logitude)`` decimal degrees',
     )
     name: str = field(default=None, doc='Place name')
-    place_type: int = field(default=None, doc='Numeric value indicating place type')
+    place_type: int = field(default=None, doc='Place type ID')
+    place_type_name: str = field(default=None, doc='Place type name')
     slug: str = field(default=None, doc='Place URL slug')
 
     @classmethod
-    def from_json(cls, value: JsonResponse, category: str = None, **kwargs) -> BaseModel:
+    def from_json(cls, value: JsonResponse, category: str = None, **kwargs) -> 'Place':
         value.setdefault('category', category)
         return super(Place, cls).from_json(value)
 
     @classmethod
-    def from_json_list(cls, value: ResponseOrResults, **kwargs) -> List[BaseModel]:
+    def from_json_list(cls, value: ResponseOrResults, **kwargs) -> List['Place']:
         """Optionally use results from /places/nearby to set Place.category"""
         json_value = dict(ensure_list(value)[0])
         if 'results' in json_value:
@@ -65,7 +66,9 @@ class Place(BaseModel):
 
         if 'standard' in json_value and 'community' in json_value:
             places = [cls.from_json(item, category='standard') for item in json_value['standard']]
-            places += [cls.from_json(item, category='community') for item in json_value['community']]
+            places += [
+                cls.from_json(item, category='community') for item in json_value['community']
+            ]
             return places
         else:
             return super(Place, cls).from_json_list(json_value)
