@@ -3,8 +3,6 @@ from typing import Dict, List
 from attr import fields_dict
 
 from pyinaturalist.constants import (
-    CONSERVATION_STATUSES,
-    ESTABLISTMENT_MEANS,
     ICONIC_EMOJI,
     ICONIC_TAXA_BASE_URL,
     INAT_BASE_URL,
@@ -17,75 +15,16 @@ from pyinaturalist.docs import EMOJI
 from pyinaturalist.models import (
     BaseModel,
     BaseModelCollection,
+    ConservationStatus,
+    EstablishmentMeans,
     LazyProperty,
+    ListedTaxon,
     Photo,
-    Place,
-    User,
     datetime_field,
     define_model,
     define_model_collection,
     field,
-    upper,
 )
-from pyinaturalist.v1 import get_taxa_by_id
-
-
-# TODO: Some more properties to consolidate differences between endpoints? (place_id vs. place, etc.)
-# TODO: Include codes from other sources? Currently only including IUCN codes.
-@define_model
-class ConservationStatus(BaseModel):
-    """:fa:`exclamation-triangle` The conservation status of a taxon in a given location, based on the schema of:
-
-    * ``Taxon.conservation_status`` from `GET /taxa <https://api.inaturalist.org/v1/docs/#!/Taxa/get_taxa>`_
-    * ``Observation.taxon.conservation_statuses`` from `GET /observations <https://api.inaturalist.org/v1/docs/#!/Observations/get_observations>`_
-    """
-
-    authority: str = field(default=None, doc='Data source for conservation status')
-    created_at: DateTime = datetime_field(doc='Date and time the conservation status was created')
-    description: str = field(default=None, doc='Description of conservation status')
-    geoprivacy: str = field(
-        default=None, doc='Default geoprivacy level; may be obscured or private for protected species'
-    )
-    iucn: int = field(default=None, doc='IUCD ID, if applicable')
-    place_id: int = field(default=None)
-    source_id: int = field(default=None)
-    status: str = field(
-        default=None,
-        converter=upper,
-        options=CONSERVATION_STATUSES,
-        doc='Short code for conservation status',
-    )
-    status_name: str = field(default=None, doc='Full name of conservation status')
-    taxon_id: int = field(default=None, doc='Taxon ID')
-    updated_at: DateTime = datetime_field(doc='Date and time the conservation status was last updated')
-    updater_id: int = field(default=None)
-    url: str = field(default=None, doc='Link to data source with more details')
-
-    # Lazy-loaded nested model objects
-    place: property = LazyProperty(
-        Place.from_json, type=Place, doc='Location that the conservation status applies to'
-    )
-    updater: property = LazyProperty(
-        User.from_json, type=User, doc='User that last updated the conservation status'
-    )
-    user: property = LazyProperty(
-        User.from_json, type=User, doc='User that created the conservation status'
-    )
-
-
-@define_model
-class EstablishmentMeans(BaseModel):
-    """:fa:`exclamation-triangle` The establishment means for a taxon in a given location"""
-
-    establishment_means: str = field(
-        default=None, options=ESTABLISTMENT_MEANS, doc='Establishment means description'
-    )
-    place: property = LazyProperty(
-        Place.from_json_list, type=Place, doc='Location that the establishment means applies to'
-    )
-
-    def __str__(self) -> str:
-        return self.establishment_means
 
 
 @define_model
@@ -111,6 +50,7 @@ class Taxon(BaseModel):
         factory=list, doc='Taxon IDs of taxa that are accepted synonyms'
     )
     extinct: bool = field(default=None, doc='Indicates if the taxon is extinct')
+    gbif_id: int = field(default=None, doc='GBIF taxon ID')
     iconic_taxon_id: int = field(
         default=0, doc='ID of the iconic taxon (e.g., general taxon "category")'
     )
@@ -120,7 +60,9 @@ class Taxon(BaseModel):
     is_active: bool = field(
         default=None, doc='Indicates if the taxon is active (and not renamed, moved, etc.)'
     )
-    listed_taxa: List[int] = field(factory=list, doc='Listed taxon IDs')
+    listed_places: bool = field(
+        default=None, doc='Indicates if there are listed places for this taxon'
+    )
     listed_taxa_count: int = field(
         default=None, doc='Number of listed taxa from this taxon + descendants'
     )
@@ -135,7 +77,9 @@ class Taxon(BaseModel):
         default=None, doc='Total number of observations of this taxon and its descendants'
     )
     parent_id: int = field(default=None, doc='Taxon ID of immediate ancestor')
-    preferred_common_name: str = field(default='', doc='Common name for the preferred place, if any')
+    preferred_common_name: str = field(
+        default='', doc='Common name for the preferred place, if any'
+    )
     preferred_establishment_means: str = field(
         default=None, doc='Establishment means for this taxon in the given preferred place (if any)'
     )
@@ -143,16 +87,15 @@ class Taxon(BaseModel):
         default=None,
         doc='Number indicating rank level, for easier comparison between ranks (kingdom=higest)',
     )
+    ranges: bool = field(default=None, doc='Indicates if there is range data for this taxon')
     rank: str = field(default=None, options=RANKS, doc='Taxon rank')
     taxon_changes_count: int = field(default=None, doc='Number of curator changes to this taxon')
     taxon_schemes_count: int = field(default=None, doc='Taxon schemes that include this taxon')
     vision: bool = field(
         default=None, doc='Indicates if this taxon is included in the computer vision model'
     )
-    wikipedia_summary: str = field(
-        default=None, doc='Taxon summary from Wikipedia article, if available'
-    )
-    wikipedia_url: str = field(default=None, doc='URL to Wikipedia article for the taxon, if available')
+    wikipedia_summary: str = field(default=None, doc='Taxon summary from Wikipedia article')
+    wikipedia_url: str = field(default=None, doc='URL to Wikipedia article for the taxon')
 
     # Lazy-loaded model objects
     ancestors: property = LazyProperty(BaseModel.from_json_list)
@@ -173,6 +116,11 @@ class Taxon(BaseModel):
         type=EstablishmentMeans,
         doc='Establishment means for a taxon in a given location',
     )
+    listed_taxa: property = LazyProperty(
+        ListedTaxon.from_json_list,
+        type=List[ListedTaxon],
+        doc='Details about this taxon associated with a list',
+    )
     taxon_photos: property = LazyProperty(
         Photo.from_json_list, type=List[Photo], doc='All taxon photos shown on taxon info page'
     )
@@ -180,7 +128,7 @@ class Taxon(BaseModel):
     # Unused attributes
     # atlas_id: int = field(default=None)
     # flag_counts: Dict[str, int] = field(factory=dict)  # {"unresolved": 1, "resolved": 2}
-    # min_species_ancestry: str = field(default=None)  #: Used internally by iNaturalist for Elasticsearch aggregations
+    # min_species_ancestry: str = field(default=None)  # Used internally by iNaturalist for Elasticsearch aggregations
     # min_species_taxon_id: int = field(default=None)
     # partial: bool = field(default=None, repr=False)
     # photos_locked: bool = field(default=None)
@@ -191,7 +139,7 @@ class Taxon(BaseModel):
         """Sort Taxon objects by rank then by name"""
         taxa = cls.from_json_list(value)
         taxa.sort(key=_get_rank_name_idx)
-        return taxa  # type: ignore
+        return taxa
 
     @property
     def ancestry(self) -> str:
@@ -225,9 +173,14 @@ class Taxon(BaseModel):
 
     @property
     def icon_url(self) -> str:
-        """URL to the iconic taxon's icon"""
+        """URL for the iconic taxon's icon"""
         return f'{ICONIC_TAXA_BASE_URL}/{self.iconic_taxon_name.lower()}-75px.png'
 
+    @property
+    def gbif_url(self) -> str:
+        """URL for the GBIF info page for this taxon"""
+        return f'https://www.gbif.org/species/{self.gbif_id}'
+
     @property
     def parent(self) -> 'Taxon':
         """Immediate parent, if any"""
@@ -241,8 +194,10 @@ class Taxon(BaseModel):
     @classmethod
     def from_id(cls, id: int) -> 'Taxon':
         """Lookup and create a new Taxon object by ID"""
+        from pyinaturalist.v1 import get_taxa_by_id
+
         r = get_taxa_by_id(id)
-        return cls.from_json(r['results'][0])  # type: ignore
+        return cls.from_json(r['results'][0])
 
     def load_full_record(self):
         """Update this Taxon with full taxon info, including ancestors + children"""
@@ -266,8 +221,8 @@ class Taxon(BaseModel):
 
 @define_model
 class TaxonCount(Taxon):
-    """:fa:`dove,style=fas` :fa:`list` A :py:class:`.Taxon` with an associated count, used in a :py:class:`.TaxonCounts`
-    collection
+    """:fa:`dove,style=fas` :fa:`list` A :py:class:`.Taxon` with an associated count, used in a
+    :py:class:`.TaxonCounts` collection
     """
 
     count: int = field(default=0, doc='Number of observations of this taxon')
@@ -278,15 +233,17 @@ class TaxonCount(Taxon):
         if 'results' in value:
             value = value['results']
         if 'taxon' in value:
+            value = value.copy()
             value.update(value.pop('taxon'))
-        return super(TaxonCount, cls).from_json(value)  # type: ignore
+        return super(TaxonCount, cls).from_json(value)
 
     @property
     def row(self) -> TableRow:
         return {
             'ID': self.id,
             'Rank': self.rank,
-            'Name': f'{self.emoji} {self.name}',
+            'Scientific name': f'{self.emoji} {self.name}',
+            'Common name': self.preferred_common_name,
             'Count': self.count,
         }
 
@@ -297,25 +254,11 @@ class TaxonCount(Taxon):
 @define_model_collection
 class TaxonCounts(BaseModelCollection):
     """:fa:`dove,style=fas` :fa:`list` A collection of taxa with an associated counts. Used with
-    `GET /observations/species_counts <https://api.inaturalist.org/v1/docs/#!/Observations/get_observations_species_counts>`_.
+    :v1:`GET /observations/species_counts <Observations/get_observations_species_counts>`.
     as well as :py:class:`.LifeList`.
     """
 
     data: List[TaxonCount] = field(factory=list, converter=TaxonCount.from_json_list)
-    _taxon_counts: Dict[int, int] = field(default=None, init=False, repr=False)
-
-    def count(self, taxon_id: int) -> int:
-        """Get an observation count for the specified taxon and its descendants, and handle unlisted taxa.
-        **Note:** ``-1`` can be used an alias for ``count_without_taxon``.
-        """
-        # Make and cache an index of taxon IDs and observation counts
-        if self._taxon_counts is None:
-            self._taxon_counts = {t.id: t.descendant_obs_count for t in self.data}
-            self._taxon_counts[-1] = self.count_without_taxon
-        return self._taxon_counts.get(taxon_id, 0)
-
-    def __str__(self) -> str:
-        return '\n'.join([str(taxon) for taxon in self.data])
 
 
 # Since these use Taxon classmethods, they must be added after Taxon is defined
