@@ -3,16 +3,19 @@ Shared unit test-related utilities.
 Pytest will also automatically pick up any fixtures defined here.
 """
 import json
-import logging
 import os
 import re
 from inspect import Parameter, getmembers, isfunction, signature
 from os.path import join
-from unittest.mock import MagicMock
+from unittest.mock import MagicMock, patch
 
-# If ipdb is installed, register it as the default debugger
+import pytest
+from requests import Session
+
+from pyinaturalist import enable_logging
 from pyinaturalist.constants import SAMPLE_DATA_DIR
 
+# If ipdb is installed, register it as the default debugger
 try:
     import ipdb  # noqa: F401
 
@@ -37,8 +40,16 @@ MOCK_CREDS_OAUTH = {
 }
 
 # Enable logging for urllib and other external loggers
-logging.basicConfig(level='INFO')
-logging.getLogger('pyinaturalist').setLevel('DEBUG')
+enable_logging('DEBUG')
+
+
+@pytest.fixture(scope='function', autouse=True)
+def patch_cached_session():
+    """Use a regular requests.Session to disable request caching and rate-limiting during tests"""
+    with patch('pyinaturalist.session.get_local_session', return_value=Session()), patch(
+        'pyinaturalist.client.ClientSession', Session
+    ):
+        yield
 
 
 def get_module_functions(module):
@@ -73,12 +84,12 @@ def get_mock_args_for_signature(func):
     return [MagicMock()] * len(required_args)
 
 
-def _sample_data_path(filename):
+def sample_data_path(filename):
     return join(SAMPLE_DATA_DIR, filename)
 
 
 def load_sample_data(filename):
-    with open(_sample_data_path(filename), encoding='utf-8') as fh:
+    with open(sample_data_path(filename), encoding='utf-8') as fh:
         if filename.endswith('json'):
             return json.load(fh)
         else:
